All of lore.kernel.org
 help / color / mirror / Atom feed
From: Nicolas Dufresne <nicolas.dufresne@collabora.com>
To: Mauro Carvalho Chehab <mchehab@kernel.org>
Cc: kernel@collabora.com,
	Sebastian Fricke <sebastian.fricke@collabora.com>,
	linux-media@vger.kernel.org, linux-kernel@vger.kernel.org
Subject: [PATCH v2 10/23] media: h264: Sort p/b reflist using frame_num
Date: Thu, 31 Mar 2022 15:37:12 -0400	[thread overview]
Message-ID: <20220331193726.289559-11-nicolas.dufresne@collabora.com> (raw)
In-Reply-To: <20220331193726.289559-1-nicolas.dufresne@collabora.com>

In the reference list builder, frame_num refers to FrameNumWrap
in the spec, which is the same as the pic_num for frame decoding.
The same applies for long_term_pic_num and long_term_frame_idx.

Sort all type of references by frame_num so the sort can be reused
for fields reflist were the sorting is done using frame_num instead.
In short, pic_num is never actually used for building reference
lists.

Signed-off-by: Nicolas Dufresne <nicolas.dufresne@collabora.com>
Reviewed-by: Sebastian Fricke <sebastian.fricke@collabora.com>
---
 drivers/media/v4l2-core/v4l2-h264.c | 23 +++++++++++++----------
 include/media/v4l2-h264.h           |  2 --
 2 files changed, 13 insertions(+), 12 deletions(-)

diff --git a/include/media/v4l2-h264.h b/include/media/v4l2-h264.h
index 4cef717b3f18..0d9eaa956123 100644
--- a/include/media/v4l2-h264.h
+++ b/include/media/v4l2-h264.h
@@ -18,7 +18,6 @@
  * @refs.top_field_order_cnt: top field order count
  * @refs.bottom_field_order_cnt: bottom field order count
  * @refs.frame_num: reference frame number
- * @refs.pic_num: reference picture number
  * @refs.longterm: set to true for a long term reference
  * @refs: array of references
  * @cur_pic_order_count: picture order count of the frame being decoded
@@ -36,7 +35,6 @@ struct v4l2_h264_reflist_builder {
 		s32 top_field_order_cnt;
 		s32 bottom_field_order_cnt;
 		int frame_num;
-		u32 pic_num;
 		u16 longterm : 1;
 	} refs[V4L2_H264_NUM_DPB_ENTRIES];
 
diff --git a/drivers/media/v4l2-core/v4l2-h264.c b/drivers/media/v4l2-core/v4l2-h264.c
index bcf9b7774560..7e1eba03099a 100644
--- a/drivers/media/v4l2-core/v4l2-h264.c
+++ b/drivers/media/v4l2-core/v4l2-h264.c
@@ -50,7 +50,6 @@ v4l2_h264_init_reflist_builder(struct v4l2_h264_reflist_builder *b,
 		if (!(dpb[i].flags & V4L2_H264_DPB_ENTRY_FLAG_ACTIVE))
 			continue;
 
-		b->refs[i].pic_num = dpb[i].pic_num;
 		if (dpb[i].flags & V4L2_H264_DPB_ENTRY_FLAG_LONG_TERM)
 			b->refs[i].longterm = true;
 
@@ -139,15 +138,19 @@ static int v4l2_h264_p_ref_list_cmp(const void *ptra, const void *ptrb,
 	}
 
 	/*
-	 * Short term pics in descending pic num order, long term ones in
-	 * ascending order.
+	 * For frames, short term pics are in descending pic num order and long
+	 * term ones in ascending order. For fields, the same direction is used
+	 * but with frame_num (wrapped). For frames, the value of pic_num and
+	 * frame_num are the same (see formula (8-28) and (8-29)). For this
+	 * reason we can use frame_num only and share this funciton between
+	 * frames and fields reflist.
 	 */
 	if (!builder->refs[idxa].longterm)
 		return builder->refs[idxb].frame_num <
 		       builder->refs[idxa].frame_num ?
 		       -1 : 1;
 
-	return builder->refs[idxa].pic_num < builder->refs[idxb].pic_num ?
+	return builder->refs[idxa].frame_num < builder->refs[idxb].frame_num ?
 	       -1 : 1;
 }
 
@@ -173,10 +176,10 @@ static int v4l2_h264_b0_ref_list_cmp(const void *ptra, const void *ptrb,
 			return 1;
 	}
 
 	/* Long term pics in ascending pic num order. */
 	if (builder->refs[idxa].longterm)
-		return builder->refs[idxa].pic_num <
-		       builder->refs[idxb].pic_num ?
+		return builder->refs[idxa].frame_num <
+		       builder->refs[idxb].frame_num ?
 		       -1 : 1;
 
 	poca = v4l2_h264_get_poc(builder, ptra);
@@ -218,10 +221,10 @@ static int v4l2_h264_b1_ref_list_cmp(const void *ptra, const void *ptrb,
 			return 1;
 	}
 
 	/* Long term pics in ascending pic num order. */
 	if (builder->refs[idxa].longterm)
-		return builder->refs[idxa].pic_num <
-		       builder->refs[idxb].pic_num ?
+		return builder->refs[idxa].frame_num <
+		       builder->refs[idxb].frame_num ?
 		       -1 : 1;
 
 	poca = v4l2_h264_get_poc(builder, ptra);
@@ -173,10 +176,10 @@ static int v4l2_h264_b0_ref_list_cmp(const void *ptra, const void *ptrb,
 	}
 
-	/* Long term pics in ascending pic num order. */
+	/* Long term pics in ascending frame num order. */
 	if (builder->refs[idxa].longterm)
@@ -218,10 +221,10 @@ static int v4l2_h264_b1_ref_list_cmp(const void *ptra, const void *ptrb,
 	}
 
-	/* Long term pics in ascending pic num order. */
+	/* Long term pics in ascending frame num order. */
 	if (builder->refs[idxa].longterm)
-- 
2.34.1


  parent reply	other threads:[~2022-03-31 19:38 UTC|newest]

Thread overview: 66+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2022-03-31 19:37 [PATCH v2 00/23] H.264 Field Decoding Support for Frame-based Decoders Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 01/23] media: doc: Document dual use of H.264 pic_num/frame_num Nicolas Dufresne
2022-03-31 20:02   ` [PATCH v2 00/23] H.264 Field Decoding Support for Frame-based Decoders Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 02/23] media: v4l2-mem2mem: Trace on implicit un-hold Nicolas Dufresne
2022-04-02 10:47   ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 03/23] media: videobuf2-v4l2: Warn on holding buffers without support Nicolas Dufresne
2022-04-02 10:52   ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 04/23] media: h264: Avoid wrapping long_term_frame_idx Nicolas Dufresne
2022-04-02 11:01   ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 05/23] media: h264: Use v4l2_h264_reference for reflist Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 06/23] media: h264: Increase reference lists size to 32 Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 12:03   ` Ezequiel Garcia
2022-04-02 12:03     ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 07/23] media: h264: Store current picture fields Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 08/23] media: h264: Store all fields into the unordered list Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 09/23] media: v4l2: Trace calculated p/b0/b1 initial reflist Nicolas Dufresne
2022-03-31 19:37 ` Nicolas Dufresne [this message]
2022-03-31 19:37 ` [PATCH v2 11/23] media: v4l2: Reorder field reflist Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 12/23] media: rkvdec: Stop overclocking the decoder Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 11:05   ` Ezequiel Garcia
2022-04-02 11:05     ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 13/23] media: rkvdec: h264: Fix dpb_valid implementation Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 11:16   ` Ezequiel Garcia
2022-04-02 11:16     ` Ezequiel Garcia
2022-04-05 15:10     ` Nicolas Dufresne
2022-04-05 15:10       ` Nicolas Dufresne
2022-04-05 15:34       ` Ezequiel Garcia
2022-04-05 15:34         ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 14/23] media: rkvdec: h264: Fix bit depth wrap in pps packet Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 11:20   ` Ezequiel Garcia
2022-04-02 11:20     ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 15/23] media: rkvdec: h264: Validate and use pic width and height in mbs Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 11:32   ` Ezequiel Garcia
2022-04-02 11:32     ` Ezequiel Garcia
2022-04-05 15:44     ` Nicolas Dufresne
2022-04-05 15:44       ` Nicolas Dufresne
2022-04-05 16:04       ` Nicolas Dufresne
2022-04-05 16:04         ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 16/23] media: rkvdec: h264: Fix reference frame_num wrap for second field Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 11:33   ` Ezequiel Garcia
2022-04-02 11:33     ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 17/23] media: rkvdec: Enable capture buffer holding for H264 Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-04-02 11:35   ` Ezequiel Garcia
2022-04-02 11:35     ` Ezequiel Garcia
2022-03-31 19:37 ` [PATCH v2 18/23] media: rkvdec: Ensure decoded resolution fit coded resolution Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 19/23] media: rkvdec-h264: Add field decoding support Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 20/23] media: hantro: Enable HOLD_CAPTURE_BUF for H.264 Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 21/23] media: hantro: Stop using H.264 parameter pic_num Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 22/23] media: hantro: h264: Make dpb entry management more robust Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne
2022-03-31 19:37 ` [PATCH v2 23/23] media: hantro: Add H.264 field decoding support Nicolas Dufresne
2022-03-31 19:37   ` Nicolas Dufresne

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20220331193726.289559-11-nicolas.dufresne@collabora.com \
    --to=nicolas.dufresne@collabora.com \
    --cc=kernel@collabora.com \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-media@vger.kernel.org \
    --cc=mchehab@kernel.org \
    --cc=sebastian.fricke@collabora.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.